from typing import Dict, Optional, Union, Any
from dataclasses import dataclass, field

from .base_log_types import BaseLog, BaseLogParams, LogType

@dataclass
class PromptReference:
    """Reference to a prompt template.
    
    This class represents a reference to a prompt template used in AI model generations.
    
    Attributes:
        slug (str): Unique identifier for the prompt template.
        version (str): Version of the prompt template.
    
    Example:
        ```python
        # Basic prompt reference
        prompt = PromptReference(slug="qa-prompt", version="2.1.0")
        ```
    """
    slug: str
    version: str

@dataclass
class GenerationParams(BaseLogParams):
    """Parameters for creating a new generation.
    
    This class defines the parameters that can be used to create a new generation,
    either with or without a prompt reference.
    
    Attributes:
        prompt (Optional[PromptReference]): Reference to the prompt template used.
        input (Optional[str]): The input provided to the model.
        output (Optional[str]): The output generated by the model.
        variables (Optional[Dict[str, Any]]): Variables used in the prompt template.
        options (Optional[Dict[str, Any]]): Additional options for the generation.
    
    Example:
        ```python
        # Create generation parameters with a prompt reference
        params = GenerationParams(
            name="answer-generation",
            prompt=PromptReference(slug="qa-prompt", version="2.1.0"),
            input="What is the capital of France?",
            variables={"style": "concise", "language": "en"}
        )
        
        # Create generation parameters without a prompt reference
        params = GenerationParams(
            name="text-completion",
            input="Complete this sentence: The sky is",
            output="The sky is blue and vast."
        )
        ```
    """
    prompt: Optional[PromptReference] = None
    input: Optional[str] = None
    output: Optional[str] = None
    variables: Optional[Dict[str, Any]] = None
    options: Optional[Dict[str, Any]] = None

@dataclass
class Generation(BaseLog):
    """Generation class representing an AI model generation within a trace.
    
    This class tracks interactions with AI models, including inputs, outputs,
    and prompt information used for the generation.
    
    Attributes:
        prompt (Optional[PromptReference]): Reference to the prompt template used.
        input (Optional[str]): The input provided to the model.
        output (Optional[str]): The output generated by the model.
        variables (Optional[Dict[str, Any]]): Variables used in the prompt template.
        type (str): The type of log, defaults to LogType.GENERATION.
        input_tokens (Optional[int]): Number of tokens used for the input.
        output_tokens (Optional[int]): Number of tokens used for the output.
        cost (Optional[float]): Cost of the generation.
    
    Example:
        ```python
        # Create a generation with a prompt reference
        generation = trace.create_generation(
            name="answer-generation",
            prompt=PromptReference(slug="qa-prompt", version="2.1.0"),
            input="What is the capital of France?"
        )
        
        # Start the generation
        generation.start()
        
        # End the generation with output
        generation.end("The capital of France is Paris.")
        
        # Update generation metadata
        generation.update(metadata={
            "model_version": "gpt-4",
            "tokens_used": 42
        })
        ```
    """
    prompt: Optional[PromptReference] = None
    input: Optional[str] = None
    output: Optional[str] = None
    variables: Optional[Dict[str, Any]] = None
    type: str = field(default=LogType.GENERATION)
    input_tokens: Optional[int] = None
    output_tokens: Optional[int] = None
    cost: Optional[float] = None

    def start(self, input: Optional[str] = None) -> 'Generation':
        """Marks the generation as started and sets the input if provided.
        
        Args:
            input (Optional[str]): Optional input data to associate with the generation.
            
        Returns:
            Generation: The generation instance for method chaining.
            
        Example:
            ```python
            # Start a generation without input
            generation.start()
            
            # Start a generation with input
            generation.start("What is the capital of France?")
            ```
        """
        ...

    def end(self, output: Optional[Union[str, Dict[str, Any]]] = None) -> 'Generation':
        """Marks the generation as ended and sets the output if provided.
        
        Args:
            output (Optional[Union[str, Dict[str, Any]]]): Optional output data from the model.
                Can be either a string or a dictionary containing output parameters.
            
        Returns:
            Generation: The generation instance for method chaining.
            
        Example:
            ```python
            # End a generation without output
            generation.end()
            
            # End a generation with output as string
            generation.end("The capital of France is Paris.")
            
            # End a generation with output params
            generation.end({
                "output": "The capital of France is Paris.",
                "input_tokens": 10,
                "output_tokens": 10,
                "cost": 0.01
            })
            ```
        """
        ...
